@@ -2,11 +2,11 @@ use 5.010; # Perl 5.10+ needed for PDL
 use strict;
 use warnings;
 package BenchmarkAnything::Evaluations;
-# git description: v0.003-2-gf405410
+# git description: v0.004-2-g53c7b4c
 
 our $AUTHORITY = 'cpan:SCHWIGON';
 # ABSTRACT: Evaluation support for BenchmarkAnything data
-$BenchmarkAnything::Evaluations::VERSION = '0.004';
+$BenchmarkAnything::Evaluations::VERSION = '0.005';
 use PDL::Core;
 use PDL::Stats;
 use PDL::Ufunc;
@@ -17,14 +17,21 @@ sub multi_point_stats
         my ($values) = @_;
 
         my $data = pdl(@$values);
-        my $avg  = average($data);
+        my $avg  = average($data); # average
+        my $stdv = stdv($data);    # standard deviation
+        my $min  = min($data);     # min
+        my $max  = max($data);     # max
+        my $se   = se($data);      # standard error of the mean (::Stats::Basic)
         return {
-                avg         => sclr($avg),
-                stdv        => stdv($data),
-                min         => min($data),
-                max         => max($data),
-                ci_95_lower => $avg - 1.96 * se($data),
-                ci_95_upper => $avg + 1.96 * se($data),
+                count       => scalar(@$values),
+                avg         => $avg,
+                stdv        => $stdv,
+                min         => $min,
+                max         => $max,
+                se          => $se,
+                ci_95_upper => $avg + 1.96 * $se,
+                ci_95_lower => $avg - 1.96 * $se,
+                badflag     => ($avg->badflag || $stdv->badflag || $min->badflag || $max->badflag || $se->badflag),
                };
 }
 
@@ -34,14 +41,17 @@ sub transform_chartlines
         my ($chartlines, $options) = @_;
 
         my $x_key       = $options->{x_key};
+        my $x_key_short = $options->{x_key_short};
         my $x_type      = $options->{x_type};
         my $y_key       = $options->{y_key};
         my $y_type      = $options->{y_type};
         my $aggregation = $options->{aggregation};
         my $verbose     = $options->{verbose};
+        my $scalars     = $options->{scalars};
         my $debug       = $options->{debug};
         my $dropnull    = $options->{dropnull};
-        my $rawnumbers  = "";
+        my $textlines   = "";
+        my @textlines   = ();
 
         # from all chartlines collect values into buckets for the dimensions we need
         #
@@ -65,12 +75,13 @@ sub transform_chartlines
                 }
                 push @titles, $title;
 
-                my $rawline = sprintf("* %-20s - %-40s\n", $title, $NAME);
-                print STDERR $rawline if $verbose;
-                $rawnumbers.=$rawline;
+                my $rawline = sprintf("* %-20s - %-40s", $title, $NAME);
+                print STDERR $rawline."\n" if $verbose;
+                $textlines .= "$rawline\n";
 
                 print STDERR "  VALUE_IDs: ".join(",", map {$_->{VALUE_ID}} @$results)."\n" if $debug;
 
+                $VALUES{$title}{NAME} = $NAME if @$results;
         POINT:
                 foreach my $point (@$results)
                 {
@@ -82,25 +93,25 @@ sub transform_chartlines
                                 print STDERR "benchmarkanything: transform_chartlines: chartline '$title': ignore data point (missing key '$x_key'): ".Data::Dumper::Dumper($results) if $verbose;
                                 next POINT;
                         }
-                        push @{$VALUES{$title}{$x}{values}}, $y; # maybe multiple for same X - average them later
+                        push @{$VALUES{$title}{xvalues}{$x}{values}}, $y; # maybe multiple for same X - average them later
                 }
         }
 
         # statistical aggregations of multi points
-        foreach my $title (keys %VALUES)
+        foreach my $title (@titles)
         {
-                foreach my $x (keys %{$VALUES{$title}})
+                foreach my $x (sort keys %{$VALUES{$title}{xvalues}})
                 {
-                        my $multi_point_values     = $VALUES{$title}{$x}{values};
-                        $VALUES{$title}{$x}{stats} = multi_point_stats($multi_point_values);
+                        my $multi_point_values     = $VALUES{$title}{xvalues}{$x}{values};
+                        $VALUES{$title}{xvalues}{$x}{stats} = multi_point_stats($multi_point_values);
                 }
         }
 
         # find out all available x-values from all chartlines
         my %all_x;
-        foreach my $title (keys %VALUES)
+        foreach my $title (@titles)
         {
-                foreach my $x (keys %{$VALUES{$title}})
+                foreach my $x (sort keys %{$VALUES{$title}{xvalues}})
                 {
                         $all_x{$x} = 1;
                 }
@@ -116,10 +127,10 @@ sub transform_chartlines
         # drop complete chartlines if it has gaps on versions that the other chartlines provide values
         my %clean_chartlines;
         if ($dropnull) {
-                foreach my $title (keys %VALUES) {
+                foreach my $title (@titles) {
                         my $ok = 1;
                         foreach my $x (@all_x) {
-                                if (not @{$VALUES{$title}{$x}{values} || []}) {
+                                if (not @{$VALUES{$title}{xvalues}{$x}{values} || []}) {
                                         print STDERR "skip: $title (missing values for $x)\n" if $verbose;
                                         $ok = 0;
                                 }
@@ -132,19 +143,20 @@ sub transform_chartlines
         }
 
         # intermediate debug output
-        foreach my $title (keys %VALUES)
+        foreach my $title (@titles)
         {
-                foreach my $x (keys %{$VALUES{$title}})
+                foreach my $x (sort keys %{$VALUES{$title}{xvalues}})
                 {
-                        my $count = scalar @{$VALUES{$title}{$x}{values} || []} || 0;
+                        my $count = scalar @{$VALUES{$title}{xvalues}{$x}{values} || []} || 0;
                         next if not $count;
-                        my $avg   = $VALUES{$title}{$x}{stats}{avg};
-                        my $stdv  = $VALUES{$title}{$x}{stats}{stdv};
-                        my $ci95l = $VALUES{$title}{$x}{stats}{ci_95_lower};
-                        my $ci95u = $VALUES{$title}{$x}{stats}{ci_95_upper};
-                        my $rawline = sprintf("  %-20s . %-7s . (ci95l..avg..ci95u) = (%2.2f .. %2.2f .. %2.2f) +- stdv %5.2f (%3d points)\n", $title, $x, $ci95l, $avg, $ci95u, $stdv, $count);
-                        $rawnumbers .= $rawline;
-                        print STDERR $rawline if $verbose;
+                        my $avg   = $VALUES{$title}{xvalues}{$x}{stats}{avg};
+                        my $stdv  = $VALUES{$title}{xvalues}{$x}{stats}{stdv};
+                        my $ci95l = $VALUES{$title}{xvalues}{$x}{stats}{ci_95_lower};
+                        my $ci95u = $VALUES{$title}{xvalues}{$x}{stats}{ci_95_upper};
+                        my $rawline = sprintf("  %-20s . %-7s . (ci95l..avg..ci95u) = (%2.2f .. %2.2f .. %2.2f) +- stdv %5.2f (%3d points)", $title, $x, $ci95l, $avg, $ci95u, $stdv, $count);
+                        $textlines .= "$rawline\n";
+                        push @textlines, $rawline;
+                        print STDERR $rawline."\n" if $verbose;
                 }
         }
 
@@ -159,18 +171,18 @@ sub transform_chartlines
                 for (my $j=0; $j<@titles; $j++) # columns
                 {
                         my $title = $titles[$j];
-                        my $value = $VALUES{$title}{$x}{stats}{$aggregation};
+                        my $value = $VALUES{$title}{xvalues}{$x}{stats}{$aggregation};
                         # stringify to unbless from PDL, then numify for type-aware JSON
                         $value    = $value ? (0+sprintf("%6.2f", $value)) : undef;
-                        $RESULTMATRIX[0]    [0]    = $x_key       if $i == 0 && $j == 0;
-                        $RESULTMATRIX[0]    [$j+1] = $title       if $i == 0;
-                        $RESULTMATRIX[$i+1] [0]    = $x           if            $j == 0;
+                        $RESULTMATRIX[0]    [0]    = ($x_key_short // $x_key) if $i == 0 && $j == 0;
+                        $RESULTMATRIX[0]    [$j+1] = $title                   if $i == 0;
+                        $RESULTMATRIX[$i+1] [0]    = $x                       if            $j == 0;
                         $RESULTMATRIX[$i+1] [$j+1] = $value;
                 }
         }
 
         if (wantarray) {
-                return (\@RESULTMATRIX, $rawnumbers);
+                return (\@RESULTMATRIX, $textlines, \@textlines, \%VALUES);
         } else {
                 return \@RESULTMATRIX;
         }
@@ -258,7 +270,7 @@ Steffen Schwigon <ss5@renormalist.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Steffen Schwigon.
+This software is copyright (c) 2023 by Steffen Schwigon.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
